@@ -1,5 +1,7 @@
 import os, shutil
 
+from .required_assets import REQUIRED_ASSETS
+
 from localcosmos_cordova_builder.logger import get_logger
 
 # WORKDIR is the directory where node_modules and the cordova binary are installed
@@ -9,33 +11,21 @@ if not WORKDIR:
 
 CORDOVA_CLI_VERSION = '11.0.0'
 
-CORDOVA_PLUGIN_VERSIONS = {
-    "cordova-plugin-camera" : "cordova-plugin-camera@6.0.0",
-    "cordova-plugin-datepicker" : "cordova-plugin-datepicker@0.9.3",
-    "cordova-plugin-device": "cordova-plugin-device@2.0.3",
-    "cordova-plugin-dialogs" : "cordova-plugin-dialogs@2.0.2",
-    "cordova-plugin-file" : "cordova-plugin-file@6.0.2",
-    "cordova-plugin-geolocation" : "cordova-plugin-geolocation@4.1.0",
-    "cordova-plugin-network-information" : "cordova-plugin-network-information@3.0.0",
-    "cordova-plugin-splashscreen" : "cordova-plugin-splashscreen@6.0.0",
-    "cordova-plugin-statusbar" : "cordova-plugin-statusbar@3.0.0",
-    "cordova-sqlite-storage" : "cordova-sqlite-storage@6.0.0",
-    "cordova-plugin-wkwebview-file-xhr" : "cordova-plugin-wkwebview-file-xhr@3.0.0",
-}
-
-CORDOVA_PLATFORM_VERSIONS = {
+DEFAULT_CORDOVA_PLATFORM_VERSIONS = {
     "android" : "android@10.1.2",
     "ios" : "ios@6.2.0",
+    "browser" : "browser@6.0.0",
 }
 
 PLATFORM_IOS = 'ios'
 PLATFORM_ANDROID = 'android'
+PLATFORM_BROWSER = 'browser'
 
 class CordovaBuildError(Exception):
     pass
 
 
-import subprocess, os, shutil, zipfile, logging, json
+import subprocess, os, shutil, zipfile, json
 from subprocess import CalledProcessError, PIPE
 
 
@@ -64,14 +54,6 @@ class CordovaAppBuilder:
 
     debug_apk_output_path = 'platforms/android/app/build/outputs/apk/debug/app-debug.apk'
 
-    default_plugins = ['cordova-plugin-device', 'cordova-plugin-network-information', 'cordova-plugin-file',
-                       'cordova-plugin-dialogs', 'cordova-plugin-splashscreen', 'cordova-sqlite-storage',
-                       'cordova-plugin-datepicker', 'cordova-plugin-statusbar', 'cordova-plugin-camera',
-                       'cordova-plugin-geolocation']
-
-    # this might be obsolete in cordova-ios@6.2.0
-    ios_plugins = ['cordova-plugin-wkwebview-file-xhr']
-
 
     def __init__(self, meta_app_definition, _cordova_build_path, _app_build_sources_path):
 
@@ -83,8 +65,7 @@ class CordovaAppBuilder:
         # eg version/5/release/cordova
         self._cordova_build_path = _cordova_build_path
 
-        # the www content of the app, without cordova.js
-        # eg version/5/release/sources/android or version/5/release/sources/ios
+        # {settings.APP_KIT_ROOT}/{meta_app.uuid}/{meta_app.current_version}/release/sources/
         self._app_build_sources_path = _app_build_sources_path
         
         # currently settings are only used for the smtp logger
@@ -121,26 +102,17 @@ class CordovaAppBuilder:
     def _app_folder_name(self):
         return self.meta_app_definition.package_name
 
+    # {settings.APP_KIT_ROOT}/{meta_app.uuid}/{meta_app.current_version}/release/sources/www
     @property
-    def _android_sources_root(self):
-        return os.path.join(self._app_build_sources_path, 'android')
-
-    @property
-    def _android_www_path(self):
-        return os.path.join(self._android_sources_root, 'www')
-
-    @property
-    def _ios_sources_root(self):
-        return os.path.join(self._app_build_sources_path, 'ios')
-
-    @property
-    def _ios_www_path(self):
-        return os.path.join(self._ios_sources_root, 'www')
+    def _app_build_sources_www_path(self):
+        return os.path.join(self._app_build_sources_path, 'www')
 
+    # {settings.APP_KIT_ROOT}/{meta_app.uuid}/{meta_app.current_version}/release/cordova/{package_name}/
     @property
     def _app_cordova_path(self):
         return os.path.join(self._cordova_build_path, self._app_folder_name)
 
+    # {settings.APP_KIT_ROOT}/{meta_app.uuid}/{meta_app.current_version}/release/cordova/{package_name}/www/
     @property
     def _cordova_www_path(self):
         return os.path.join(self._app_cordova_path, 'www')
@@ -149,17 +121,9 @@ class CordovaAppBuilder:
     def config_xml_path(self):
         return os.path.join(self._app_cordova_path, 'config.xml')
 
-    def _custom_config_xml_path(self, platform):
-
-        filename = 'config.xml'
-
-        if platform == 'android':
-            custom_config_xml_path = os.path.join(self._android_sources_root, filename)
-
-        elif platform == 'ios':
-            custom_config_xml_path = os.path.join(self._ios_sources_root, filename)
-        
-        return custom_config_xml_path
+    @property
+    def _custom_config_xml_path(self):
+        return os.path.join(self._app_build_sources_path, 'config.xml')
 
     # installing the cordova CLI
     def load_cordova(self):
@@ -194,29 +158,36 @@ class CordovaAppBuilder:
 
     #######################################################################################################
     # blank app and plugins
+    def _get_cordova_platform_version(self, platform):
+
+        platform_version = DEFAULT_CORDOVA_PLATFORM_VERSIONS[platform]
+
+        frontend_settings = self.meta_app_definition.frontend
+        cordova_settings = frontend_settings.get('cordova', {})
+        platforms = cordova_settings.get('platforms', {})
+        
+        if platform in platforms:
+            platform_version = platforms[platform]
+
+        return platform_version
+
+
+    def _get_cordova_plugins(self):
     
-    def install_default_plugins(self):
+        frontend_settings = self.meta_app_definition.frontend
+        cordova_settings = frontend_settings.get('cordova', {})
+        plugins = cordova_settings.get('plugins', [])
+
+        return plugins
+
+    
+    def _install_cordova_plugins(self):
 
         commands = []
 
-        for plugin in self.default_plugins:
-            commands.append([self.cordova_bin, 'plugin', 'add', CORDOVA_PLUGIN_VERSIONS[plugin]])
-
-        for command in commands:
-            process_completed = subprocess.run(command, stdout=PIPE, stderr=PIPE, cwd=self._app_cordova_path)
-
-            if process_completed.returncode != 0:
-                raise CordovaBuildError(process_completed.stderr)
-
-
-    def install_specific_plugins(self, platform):
-
-        commands = []
-
-        if platform == PLATFORM_IOS:
-            for plugin in self.ios_plugins:
-                commands.append([self.cordova_bin, 'plugin', 'add', CORDOVA_PLUGIN_VERSIONS[plugin]])
-
+        plugins = self._get_cordova_plugins()
+        for plugin in plugins:
+            commands.append([self.cordova_bin, 'plugin', 'add', plugin])
 
         for command in commands:
             process_completed = subprocess.run(command, stdout=PIPE, stderr=PIPE, cwd=self._app_cordova_path)
@@ -225,7 +196,8 @@ class CordovaAppBuilder:
                 raise CordovaBuildError(process_completed.stderr)
             
 
-    def _build_blank_cordova_app(self, platform, rebuild=False):
+    # rebuild should be set to False once we are out of development
+    def _build_blank_cordova_app(self, rebuild=True):
 
         if rebuild == True:
             self.logger.info('rebuild is set to True. removing {0}'.format(self._cordova_build_path))
@@ -252,73 +224,73 @@ class CordovaAppBuilder:
             create_process_completed = subprocess.run(create_command, stdout=PIPE, stderr=PIPE,
                                                       cwd=self._cordova_build_path)
 
-
             if create_process_completed.returncode != 0:
                 raise CordovaBuildError(create_process_completed.stderr)
 
-            # add custom config.xml if any
-            custom_config_xml_path = self._custom_config_xml_path(platform=platform)
+            self.logger.info('successfully built initial blank cordova app')
 
-            if os.path.isfile(custom_config_xml_path):
-                self.logger.info('Copying custom config xml')
-                shutil.copyfile(custom_config_xml_path, self.config_xml_path)
-                
-                with open(self.config_xml_path, 'rb') as config_file:
-                    config_xml_tree = etree.parse(config_file)
-                
-                
-                # make sure widget.id and <name> are set correctly
-                # <widget xmlns="http://www.w3.org/ns/widgets" xmlns:cdv="http://cordova.apache.org/ns/1.0" id="[package_name]" version="5.0.385">
-                # <name>[self.meta_app_definition.name]</name>
-                
-                root = xml_tree.getroot()
-                root.attrib['id'] = package_name
-                
-                for child in root:
-                    tag_name = etree.QName(child.tag).localname
-                    if tag_name == 'name':
-                        child.text = self.meta_app_definition.name
-                        break
-                
-                with open(self.config_xml_path, 'wb') as config_file:
-                    xml_tree.write(config_file, encoding='utf-8', xml_declaration=True, pretty_print=True)
-                
-                
-
-                
-            self.install_default_plugins()
-
-            self.install_specific_plugins(platform)
-
-            # add stuff to config
-            # <preference name="SplashMaintainAspectRatio" value="true" />
-            # <preference name="StatusBarStyle" value="blackopaque" />
-            # <preference name="StatusBarOverlaysWebView" value="false" />
-            # <preference name="StatusBarBackgroundColor" value="#000000" />
-
-            preferences = [
-                {'name' : 'SplashMaintainAspectRatio', 'value' : 'true'},
-                {'name' : 'StatusBarStyle', 'value' : 'blackopaque'},
-                {'name' : 'StatusBarOverlaysWebView', 'value' : 'false'},
-                {'name' : 'StatusBarBackgroundColor', 'value' : '#000000'},
-                {'name' : 'DisallowOverscroll', 'value': 'true'},
-            ]
-
-            for tag_attributes in preferences:
-                self._add_to_config_xml('preference', tag_attributes=tag_attributes)
             
+    def _update_config_xml(self):
+
+        package_name = self.meta_app_definition.package_name
+
+        # add custom config.xml if any
+        custom_config_xml_path = self._custom_config_xml_path
+
+        if os.path.isfile(custom_config_xml_path):
+            self.logger.info('Copying custom config xml')
+            shutil.copyfile(custom_config_xml_path, self.config_xml_path)                
+            
+            # make sure widget.id and <name> are set correctly
+            # <widget xmlns="http://www.w3.org/ns/widgets" xmlns:cdv="http://cordova.apache.org/ns/1.0" id="[package_name]" version="5.0.385">
+            # <name>[self.meta_app_definition.name]</name>
+
+            with open(self.config_xml_path, 'rb') as config_file:
+                xml_tree = etree.parse(config_file)
+            
+            root = xml_tree.getroot()
+            root.attrib['id'] = package_name
+            
+            for child in root:
+                tag_name = etree.QName(child.tag).localname
+                if tag_name == 'name':
+                    child.text = self.meta_app_definition.name
+                    break
+            
+            with open(self.config_xml_path, 'wb') as config_file:
+                xml_tree.write(config_file, encoding='utf-8', xml_declaration=True, pretty_print=True)
+
+        # add stuff to config
+        # <preference name="SplashMaintainAspectRatio" value="true" />
+        # <preference name="StatusBarStyle" value="blackopaque" />
+        # <preference name="StatusBarOverlaysWebView" value="false" />
+        # <preference name="StatusBarBackgroundColor" value="#000000" />
+
+        preferences = [
+            {'name' : 'SplashMaintainAspectRatio', 'value' : 'true'},
+            {'name' : 'StatusBarStyle', 'value' : 'blackopaque'},
+            {'name' : 'StatusBarOverlaysWebView', 'value' : 'false'},
+            {'name' : 'StatusBarBackgroundColor', 'value' : '#000000'},
+            {'name' : 'DisallowOverscroll', 'value': 'true'},
+        ]
+
+        for tag_attributes in preferences:
+            self._add_to_config_xml('preference', tag_attributes=tag_attributes)
+        
     
     #####################################################################################################
     # add WWW
     # determine if the www folder already is the apps one: check for www/settings,json
 
-    def _add_cordova_www_folder(self, source_www_path):
+    def _add_cordova_www_folder(self):
 
         self.logger.info('Adding app www, removing if already exists')
 
         if os.path.isdir(self._cordova_www_path):
             shutil.rmtree(self._cordova_www_path)
 
+        source_www_path = os.path.join(self._app_build_sources_path, 'www')
+
         # copy common www, cordova cannot work with symlinks
         shutil.copytree(source_www_path, self._cordova_www_path)
         
@@ -411,13 +383,18 @@ class CordovaAppBuilder:
 
         self.load_cordova()
 
-        self._build_blank_cordova_app(PLATFORM_ANDROID, rebuild=rebuild)
+        self._build_blank_cordova_app(rebuild=rebuild)
+
+        self._update_config_xml()
+
+        self._install_cordova_plugins()
 
         # set app version
         self.set_config_xml_app_version(self.meta_app_definition.current_version, self.build_number)
 
         self.logger.info('Adding android platform')
-        add_android_command = [self.cordova_bin, 'platform', 'add', CORDOVA_PLATFORM_VERSIONS['android']]
+        platform_version = self._get_cordova_platform_version(PLATFORM_ANDROID)
+        add_android_command = [self.cordova_bin, 'platform', 'add', platform_version]
 
 
         add_android_completed_process = subprocess.run(add_android_command, stdout=PIPE, stderr=PIPE,
@@ -427,16 +404,16 @@ class CordovaAppBuilder:
             raise CordovaBuildError(add_android_completed_process.stderr)
         
         # replace cordova default www with android www
-        self._add_cordova_www_folder(self._android_www_path)
+        self._add_cordova_www_folder()
 
         # build android images
         self.logger.info('building Android launcher and splashscreen images')
         image_creator = AndroidAppImageCreator(self.meta_app_definition, self._app_cordova_path,
                                                 self._app_build_sources_path)
         
-        image_creator.generate_images_from_svg('launcherIcon')
-        image_creator.generate_images_from_svg('launcherBackground')
-        image_creator.generate_images_from_svg('splashscreen', varying_ratios=True)
+        for image_type, image_filename in REQUIRED_ASSETS.items():
+            image_creator.generate_images_from_svg(image_type)
+        
 
         self.logger.info('initiating cordova build android for release aab')
         build_android_command = [self.cordova_bin, 'build', 'android', '--release', '--',
@@ -463,7 +440,6 @@ class CordovaAppBuilder:
         if build_android_apk_process_completed.returncode != 0:
             raise CordovaBuildError(build_android_apk_process_completed.stderr)
 
-
         return self._aab_filepath, self._apk_filepath
 
 
@@ -582,7 +558,6 @@ class CordovaAppBuilder:
     # res folder lies in the same folder as www
     def set_config_xml_storyboard_images(self):
 
-
         attributes_2x = {
             'src' : 'res/screen/ios/Default@2x~universal~anyany.png'
         }
@@ -600,7 +575,11 @@ class CordovaAppBuilder:
         
         self.load_cordova()
 
-        self._build_blank_cordova_app(PLATFORM_IOS, rebuild=rebuild)
+        self._build_blank_cordova_app(rebuild=rebuild)
+
+        self._update_config_xml()
+
+        self._install_cordova_plugins()
         
         # set app version
         self.set_config_xml_app_version(self.meta_app_definition.current_version, self.build_number)
@@ -623,7 +602,8 @@ class CordovaAppBuilder:
         # self.config_xml_enable_wkwebview()
 
         self.logger.info('Adding ios platform')
-        add_ios_command = [self.cordova_bin, 'platform', 'add', CORDOVA_PLATFORM_VERSIONS['ios']]
+        platform_version = self._get_cordova_platform_version(PLATFORM_IOS)
+        add_ios_command = [self.cordova_bin, 'platform', 'add', platform_version]
 
         add_ios_completed_process = subprocess.run(add_ios_command, stdout=PIPE, stderr=PIPE,
                                                    cwd=self._app_cordova_path)
@@ -633,15 +613,15 @@ class CordovaAppBuilder:
                 raise CordovaBuildError(add_ios_completed_process.stderr)
 
         # replace default cordova www folder with ios www
-        self._add_cordova_www_folder(self._ios_www_path)
+        self._add_cordova_www_folder()
 
         # build ios images
         self.logger.info('building iOS launcher and splashscreen images')
         image_creator = IOSAppImageCreator(self.meta_app_definition, self._app_cordova_path,
                                                 self._app_build_sources_path)
-        
-        image_creator.generate_images_from_svg('launcherIcon', remove_alpha_channel=True)
-        image_creator.generate_images_from_svg('splashscreen', varying_ratios=True)
+                                                
+        for image_type, image_filename in REQUIRED_ASSETS.items():
+            image_creator.generate_images_from_svg(image_type)
 
         self.set_config_xml_storyboard_images()
         image_creator.generate_storyboard_images()
@@ -663,6 +643,84 @@ class CordovaAppBuilder:
         return self._ipa_filepath
 
 
+    ##############################################################################################################
+    # BUILD browser .zip
+    # - create blank cordova app, if not yet present
+    # - install plugins
+    # - copy config.xml and other files
+    # - copy www
+    # - run cordova build command
+    ##############################################################################################################
+
+    @property
+    def _browser_zip_filepath(self):
+        filename = '{0}.zip'.format(self.meta_app_definition.name)
+        return os.path.join(self._app_cordova_path, 'platforms/browser', filename)
+
+
+    # {settings.APP_KIT_ROOT}/{meta_app.uuid}/{meta_app.current_version}/release/cordova/{meta_app_definition.package_name}/platforms/browser/www
+    @property
+    def _browser_built_www_path(self):
+        # uses the default .apk filename created by cordova
+        return os.path.join(self._app_cordova_path, 'platforms/browser/www')
+
+    def build_browser(self, rebuild=False, build_zip=False):
+        
+        self.logger.info('Building cordova browser app')
+
+        self.load_cordova()
+
+        self._build_blank_cordova_app(rebuild=rebuild)
+
+        self._update_config_xml()
+
+        self._install_cordova_plugins()
+
+        # set app version
+        self.set_config_xml_app_version(self.meta_app_definition.current_version, self.build_number)
+
+        self.logger.info('Adding browser platform')
+        platform_version = self._get_cordova_platform_version(PLATFORM_BROWSER)
+        add_browser_command = [self.cordova_bin, 'platform', 'add', platform_version]
+
+        add_browser_completed_process = subprocess.run(add_browser_command, stdout=PIPE, stderr=PIPE,
+                                                       cwd=self._app_cordova_path)
+
+        if add_browser_completed_process.returncode != 0:
+            raise CordovaBuildError(add_browser_completed_process.stderr)
+
+        # replace cordova default www with android www
+        self._add_cordova_www_folder()
+
+        # build ios release
+        self.logger.info('initiating cordova build browser')
+
+        build_browser_command = [self.cordova_bin, 'build', 'browser', '--release']
+
+        build_browser_process_completed = subprocess.run(build_browser_command, stdout=PIPE, stderr=PIPE,
+                                                     cwd=self._app_cordova_path)
+
+        if build_browser_process_completed.returncode != 0:
+            raise CordovaBuildError(build_browser_process_completed.stderr)
+
+        if build_zip == True:
+            self.logger.info('building zip_file for browser')
+
+            zip_filepath = self._browser_zip_filepath
+
+            with zipfile.ZipFile(zip_filepath, 'w', zipfile.ZIP_DEFLATED) as www_zip:
+
+                for root, dirs, files in os.walk(self._browser_built_www_path, followlinks=True):
+
+                    for filename in files:
+                        app_file_path = os.path.join(root, filename)
+                        arcname ='www/{0}'.format(app_file_path.split(self._browser_built_www_path)[-1])
+                        www_zip.write(app_file_path, arcname=arcname)
+
+        self.logger.info('successfully built browser')
+        return self._browser_built_www_path, self._browser_zip_filepath
+
+
 
 # install a non-global (local) copy of apache cordova
 class CordovaManager:
@@ -704,6 +762,5 @@ class CordovaManager:
         cordova_install_command_result = subprocess.run(cordova_install_command, stdout=PIPE, stderr=PIPE,
                                                         cwd=WORKDIR)
 
-
         if cordova_install_command_result.returncode != 0:
             raise CordovaBuildError(cordova_install_command_result.stderr)
